% Test Opensoftmax

format long; format compact

% Sample problem
K=3; D=100; xs=10*ones(D,1); A=randn(K,D); b=randn(K,1);

% Solution for a single l
l=1; rho=0.5; tic; [X,E,G,a] = Opensoftmax(xs,1,l,A,b,[],[],[],rho);toc

figure(1); plot(E,'o-'); xlabel('its'); ylabel('E');
figure(2); [order,M] = convseq(E,E(end),[length(E)-5 length(E)-1]);
figure(3); [order,M] = convseq(X,X(end,:),[length(E)-5 length(E)-1]);
figure(4); plot(1+log(a)/log(rho),'o-'); xlabel('its'); ylabel('#l.s. trials');

E,G

% An example showing precision loss (can't reach norm(g)<1e-6 and oscillates):
% rng(11); K=13; D=1000; xs=10*ones(D,1); A=randn(K,D); b=randn(K,1);
% l=100; [X,E,G,a] = Opensoftmax(xs,1,l,A,b,xs,1e-6,[],0.8); a,G,E

% Example where some values are fixed (the odd indices):
I = 1:2:D; y = sqrt(I)';
J = setdiff(1:D,I); [X,E,G,a] = Opensoftmax(xs(J),1,l,A(:,J),A(:,I)*y+b);
x = xs; x(I) = y; x(J) = X(end,:)';

% Path over l, warm-started from the previous l
tic;
X = xs';
for l=10.^(2:-1:-5)
  [X,E,G,a] = Opensoftmax(xs,1,l,A,b,X(end,:)'); a
end
toc

