% function f = Fsoftmax(X,W,w[,o]) Softmax
%
% f(x) = softmax(x,W,v)
% f has K dimensions, the softmax values (one per class), which sum to 1. If
% c=1 we return instead the index of the winner class (argmax of the K values).
%
% In:
%   X: NxD list of row vectors.
%   W: KxD matrix.
%   w: Kx1 vector.
%   o: 0 to return the K softmax values, 1 to return the index of the winner
%      class. Default: 0.
% Out:
%   f: Nx? list of K softmax values (if c=0) or their argmax (if c=1).

% Copyright (c) 2020 by Miguel A. Carreira-Perpinan

function f = Fsoftmax(X,W,w,o)

f = bsxfun(@plus,X*W',w');
f = exp(bsxfun(@minus,f,max(f,[],2)));	% Avoid underflow in softmax
f = bsxfun(@rdivide,f,sum(f,2));
if exist('o','var') && (o==1) [~,f] = max(f,[],2); end

  